@@ -19,7 +19,6 @@ from automagik.agents.models.automagik_agent import AutomagikAgent
 from automagik.agents.models.dependencies import AutomagikAgentsDependencies
 from automagik.agents.models.response import AgentResponse
 from automagik.memory.message_history import MessageHistory
-from automagik.db import get_session, update_session
 
 # Import execution components
 from .executor_factory import ExecutorFactory
@@ -27,6 +26,9 @@ from .models import ClaudeCodeRunRequest, ClaudeCodeRunResponse
 from .log_manager import get_log_manager
 from .utils import get_current_git_branch_with_fallback
 
+# Import tracing
+from automagik.tracing import get_tracing_manager
+
 logger = logging.getLogger(__name__)
 
 
@@ -94,7 +96,7 @@ class ClaudeCodeAgent(AutomagikAgent):
         # Register default tools (not applicable for local execution)
         # Tools are managed via workflow configurations
         
-        logger.info(f"ClaudeCodeAgent initialized successfully in local mode")
+        logger.debug("ClaudeCodeAgent initialized successfully in local mode")
     
     async def run(self, input_text: str, *, multimodal_content=None, 
                  system_message=None, message_history_obj: Optional[MessageHistory] = None,
@@ -337,8 +339,9 @@ class ClaudeCodeAgent(AutomagikAgent):
             
             # Check for required workflow files and validate JSON files
             # Note: prompt.md is optional - when missing, workflow uses default Claude behavior
-            required_files = [".mcp.json", "allowed_tools.json"]
-            optional_files = ["prompt.md"]
+            # .mcp.json is also optional - will use root project .mcp.json as fallback
+            required_files = ["allowed_tools.json"]
+            optional_files = ["prompt.md", ".mcp.json", "config.json"]
             
             for required_file in required_files:
                 file_path = os.path.join(workflow_path, required_file)
@@ -350,7 +353,7 @@ class ClaudeCodeAgent(AutomagikAgent):
             for optional_file in optional_files:
                 file_path = os.path.join(workflow_path, optional_file)
                 if not os.path.exists(file_path):
-                    logger.info(f"Optional workflow file missing (will use default behavior): {file_path}")
+                    logger.debug(f"Optional workflow file missing (will use default behavior): {file_path}")
             
             # Validate JSON files for both required and optional files that exist
             all_files = required_files + optional_files
@@ -428,7 +431,7 @@ class ClaudeCodeAgent(AutomagikAgent):
                             "valid": False
                         }
             
-            logger.info(f"Found {len(workflows)} workflows")
+            logger.debug(f"Found {len(workflows)} workflows")
             return workflows
             
         except Exception as e:
@@ -675,6 +678,53 @@ class ClaudeCodeAgent(AutomagikAgent):
         # Track execution timing for message persistence
         start_time = time.time()
         
+        # Initialize tracing
+        tracing = get_tracing_manager()
+        trace_id = str(uuid.uuid4())
+        root_span_id = str(uuid.uuid4())
+        langwatch_provider = None
+        
+        # Get LangWatch provider if available
+        if tracing and tracing.observability:
+            for provider in tracing.observability.providers.values():
+                if hasattr(provider, 'log_metadata'):
+                    langwatch_provider = provider
+                    break
+        
+        # Helper function for logging to LangWatch
+        def log_workflow_trace(event_type: str, span_id: str, parent_span_id: Optional[str] = None, 
+                              name: Optional[str] = None, attributes: Optional[Dict[str, Any]] = None):
+            """Log workflow trace to LangWatch if available."""
+            if langwatch_provider:
+                langwatch_provider.log_metadata({
+                    "trace_id": trace_id,
+                    "span_id": span_id,
+                    "parent_span_id": parent_span_id,
+                    "event_type": event_type,
+                    "name": name or f"claude_code.workflow.{workflow_name}",
+                    "attributes": attributes or {},
+                    "timestamp": datetime.utcnow().isoformat()
+                })
+        
+        # Start root trace for workflow execution
+        log_workflow_trace(
+            event_type="trace_start",
+            span_id=root_span_id,
+            attributes={
+                "workflow_name": workflow_name,
+                "run_id": run_id,
+                "session_id": session_id,
+                "user_id": kwargs.get("user_id"),
+                "git_branch": kwargs.get("git_branch"),
+                "repository_url": kwargs.get("repository_url"),
+                "persistent": kwargs.get("persistent", True),
+                "temp_workspace": kwargs.get("temp_workspace", False),
+                "auto_merge": kwargs.get("auto_merge", False),
+                "max_turns": kwargs.get("max_turns"),
+                "input_length": len(input_text)
+            }
+        )
+        
         try:
             # Look up existing Claude session ID from database if session_id provided
             claude_session_id_for_resumption = None
@@ -776,6 +826,24 @@ class ClaudeCodeAgent(AutomagikAgent):
             
             # Create workspace for this workflow run
             workspace_path = None
+            workspace_span_id = str(uuid.uuid4())
+            
+            # Log workspace creation span
+            log_workflow_trace(
+                event_type="span_start",
+                span_id=workspace_span_id,
+                parent_span_id=root_span_id,
+                name="claude_code.workspace.creation",
+                attributes={
+                    "repository_url": request.repository_url,
+                    "git_branch": request.git_branch,
+                    "persistent": request.persistent,
+                    "temp_workspace": request.temp_workspace
+                }
+            )
+            
+            workspace_start_time = time.time()
+            
             if hasattr(self.executor, 'environment_manager') and self.executor.environment_manager:
                 # Use prepare_workspace for external repositories, create_workspace for local worktrees
                 if request.repository_url:
@@ -798,23 +866,55 @@ class ClaudeCodeAgent(AutomagikAgent):
                         git_branch=request.git_branch
                     )
                     logger.info(f"Created workspace for run {run_id}: {workspace_path}")
-                
-                # Update workflow run with workspace path
-                if workspace_path and run_id:
-                    try:
-                        from automagik.db.models import WorkflowRunUpdate
-                        from automagik.db.repository.workflow_run import update_workflow_run_by_run_id
-                        
-                        update_data = WorkflowRunUpdate(
-                            workspace_path=str(workspace_path)
-                        )
-                        update_workflow_run_by_run_id(run_id, update_data)
-                        logger.info(f"Updated workflow run {run_id} with workspace path: {workspace_path}")
-                    except Exception as e:
-                        logger.error(f"Failed to update workspace path: {e}")
+            
+            # Log workspace creation completion
+            log_workflow_trace(
+                event_type="span_end",
+                span_id=workspace_span_id,
+                parent_span_id=root_span_id,
+                name="claude_code.workspace.creation",
+                attributes={
+                    "workspace_path": str(workspace_path) if workspace_path else None,
+                    "duration_ms": (time.time() - workspace_start_time) * 1000
+                }
+            )
+            
+            # Update workflow run with workspace path
+            if workspace_path and run_id:
+                try:
+                    from automagik.db.models import WorkflowRunUpdate
+                    from automagik.db.repository.workflow_run import update_workflow_run_by_run_id
+                    
+                    update_data = WorkflowRunUpdate(
+                        workspace_path=str(workspace_path)
+                    )
+                    update_workflow_run_by_run_id(run_id, update_data)
+                    logger.info(f"Updated workflow run {run_id} with workspace path: {workspace_path}")
+                except Exception as e:
+                    logger.error(f"Failed to update workspace path: {e}")
 
             # Execute the workflow - use standard execution to avoid SDK TaskGroup issues
             # The SDK executor can extract data from the result without streaming complications
+            execution_span_id = str(uuid.uuid4())
+            
+            # Log Claude execution span
+            log_workflow_trace(
+                event_type="span_start",
+                span_id=execution_span_id,
+                parent_span_id=root_span_id,
+                name="claude_code.execution",
+                attributes={
+                    "workflow_name": workflow_name,
+                    "session_id": session_id,
+                    "run_id": run_id,
+                    "workspace": str(workspace_path) if workspace_path else ".",
+                    "max_turns": request.max_turns,
+                    "timeout": request.timeout
+                }
+            )
+            
+            execution_start_time = time.time()
+            
             result = await self.executor.execute_claude_task(
                 request=request,
                 agent_context={
@@ -823,7 +923,26 @@ class ClaudeCodeAgent(AutomagikAgent):
                     "run_id": run_id,  # Ensure run_id is always present for logging
                     "db_id": self.db_id,
                     "workspace": str(workspace_path) if workspace_path else ".",  # Add workspace path
-                    "user_id": kwargs.get("user_id")  # Pass user_id for temp workspace creation
+                    "user_id": kwargs.get("user_id"),  # Pass user_id for temp workspace creation
+                    "trace_id": trace_id,  # Pass trace context for SDK executor
+                    "parent_span_id": execution_span_id
+                }
+            )
+            
+            # Log Claude execution completion
+            log_workflow_trace(
+                event_type="span_end",
+                span_id=execution_span_id,
+                parent_span_id=root_span_id,
+                name="claude_code.execution",
+                attributes={
+                    "success": result.get("success", False),
+                    "claude_session_id": result.get("session_id"),
+                    "turns_used": result.get("turn_count", 0),
+                    "total_tokens": result.get("token_details", {}).get("total_tokens", 0),
+                    "cost_usd": result.get("cost_usd", 0.0),
+                    "duration_ms": (time.time() - execution_start_time) * 1000,
+                    "tools_used": result.get("tools_used", [])
                 }
             )
             
@@ -1091,26 +1210,84 @@ class ClaudeCodeAgent(AutomagikAgent):
                     logger.error(f"📝 AUTO-COMMIT: 💥 EXCEPTION for run {run_id}: {commit_error}")
                     # Don't fail the workflow for commit errors
             else:
-                logger.info(f"📝 AUTO-COMMIT: ⏭️  SKIPPED - Conditions not met")
+                logger.info("📝 AUTO-COMMIT: ⏭️  SKIPPED - Conditions not met")
             
         except Exception as e:
             logger.error(f"Error in background workflow execution: {str(e)}")
             
-            # Update session with error status
-            try:
-                from automagik.db import get_session, update_session
-                session_obj = get_session(uuid.UUID(session_id))
-                if session_obj:
-                    metadata = session_obj.metadata or {}
-                    metadata.update({
-                        "run_status": "failed",
-                        "error": str(e),
-                        "completed_at": datetime.utcnow().isoformat(),
-                    })
-                    session_obj.metadata = metadata
-                    update_session(session_obj)
-            except Exception as update_error:
-                logger.error(f"Failed to update session status: {update_error}")
+            # Log trace completion with error
+            log_workflow_trace(
+                event_type="trace_end",
+                span_id=root_span_id,
+                attributes={
+                    "success": False,
+                    "error": str(e),
+                    "error_type": type(e).__name__,
+                    "duration_ms": (time.time() - start_time) * 1000
+                }
+            )
+            raise
+        finally:
+            # Log successful trace completion
+            if 'e' not in locals():
+                log_workflow_trace(
+                    event_type="trace_end",
+                    span_id=root_span_id,
+                    attributes={
+                        "success": result.get("success", False) if 'result' in locals() else False,
+                        "duration_ms": (time.time() - start_time) * 1000,
+                        "final_status": "completed" if ('result' in locals() and result.get("success", False)) else "failed",
+                        "total_tokens": result.get("token_details", {}).get("total_tokens", 0) if 'result' in locals() else 0,
+                        "cost_usd": result.get("cost_usd", 0.0) if 'result' in locals() else 0.0,
+                        "auto_commit": kwargs.get('auto_merge', False),
+                        "workspace_path": str(workspace_path) if 'workspace_path' in locals() and workspace_path else None
+                    }
+                )
+            
+            # Update workflow_runs table with failure status if there was an error
+            if 'e' in locals():
+                try:
+                    from automagik.db.models import WorkflowRunUpdate
+                    from automagik.db.repository.workflow_run import update_workflow_run_by_run_id
+                    
+                    update_data = WorkflowRunUpdate(
+                        status="failed",
+                        error_message=str(e),
+                        completed_at=datetime.utcnow(),
+                        updated_at=datetime.utcnow()
+                    )
+                    update_workflow_run_by_run_id(run_id, update_data)
+                    logger.info(f"Updated workflow run {run_id} status to failed: {str(e)}")
+                except Exception as workflow_update_error:
+                    logger.error(f"Failed to update workflow run status: {workflow_update_error}")
+                
+                # Update session with error status
+                if 'session_id' in locals() and session_id:
+                    try:
+                        from automagik.db import get_session, update_session
+                        # Check if session_id is already a UUID or needs conversion
+                        if isinstance(session_id, str):
+                            try:
+                                session_uuid = uuid.UUID(session_id)
+                            except ValueError:
+                                logger.error(f"Invalid session_id format: {session_id}")
+                                session_uuid = None
+                        else:
+                            session_uuid = session_id
+                            
+                        if session_uuid:
+                            session_obj = get_session(session_uuid)
+                            if session_obj:
+                                metadata = session_obj.metadata or {}
+                                metadata.update({
+                                    "run_status": "failed",
+                                    "error": str(e),
+                                    "completed_at": datetime.utcnow().isoformat(),
+                                })
+                                session_obj.metadata = metadata
+                                update_session(session_obj)
+                    except Exception as update_error:
+                        logger.error(f"Failed to update session status: {update_error}")
     
     async def get_run_status(self, run_id: str) -> Dict[str, Any]:
         """Get the status of an async run.
