@@ -10,7 +10,7 @@ import time
 import traceback
 from datetime import datetime
 from pathlib import Path
-from typing import Dict, Any
+from typing import Dict, Any, List
 from uuid import uuid4
 
 from claude_code_sdk import query, ClaudeCodeOptions
@@ -21,6 +21,9 @@ from .sdk_process_manager import ProcessManager
 from .sdk_metrics_handler import MetricsHandler
 from .log_manager import get_log_manager
 
+# Import tracing
+from automagik.tracing import get_tracing_manager
+
 logger = logging.getLogger(__name__)
 
 
@@ -139,6 +142,119 @@ class ExecutionStrategies:
         
         return options
     
+    async def _check_and_process_pending_messages(
+        self, 
+        workspace_path: Path, 
+        run_id: str
+    ) -> List[str]:
+        """
+        Check for pending injected messages and process them.
+        
+        Args:
+            workspace_path: Path to the workflow workspace
+            run_id: The workflow run ID
+            
+        Returns:
+            List of messages to inject into the conversation
+        """
+        try:
+            message_queue_file = workspace_path / ".pending_messages.json"
+            
+            if not message_queue_file.exists():
+                return []
+            
+            # Read and parse pending messages
+            import json
+            try:
+                with open(message_queue_file, 'r') as f:
+                    pending_messages = json.load(f)
+                
+                if not isinstance(pending_messages, list):
+                    return []
+                    
+            except (json.JSONDecodeError, IOError) as e:
+                logger.warning(f"Failed to read pending messages file: {e}")
+                return []
+            
+            # Filter unprocessed messages
+            unprocessed_messages = [
+                msg for msg in pending_messages 
+                if not msg.get("processed", False)
+            ]
+            
+            if not unprocessed_messages:
+                return []
+            
+            # Extract messages to inject
+            messages_to_inject = []
+            for msg in unprocessed_messages:
+                messages_to_inject.append(msg["message"])
+                # Mark as processed
+                msg["processed"] = True
+                msg["processed_at"] = datetime.utcnow().isoformat()
+            
+            # Write back the updated queue with processed flags
+            try:
+                with open(message_queue_file, 'w') as f:
+                    json.dump(pending_messages, f, indent=2)
+                
+                logger.info(f"Processed {len(messages_to_inject)} injected messages for run {run_id}")
+                
+            except IOError as e:
+                logger.error(f"Failed to update message queue file: {e}")
+                # Still return the messages even if we can't update the file
+            
+            return messages_to_inject
+            
+        except Exception as e:
+            logger.error(f"Error checking pending messages for run {run_id}: {e}")
+            return []
+    
+    async def _build_enhanced_prompt_with_injected_messages(
+        self,
+        original_prompt: str,
+        collected_messages: List[Any],
+        injected_messages: List[str]
+    ) -> str:
+        """
+        Build an enhanced prompt that includes conversation history and injected messages.
+        
+        Args:
+            original_prompt: The original user prompt
+            collected_messages: Messages collected so far in the conversation
+            injected_messages: New messages to inject
+            
+        Returns:
+            Enhanced prompt string
+        """
+        try:
+            # Build conversation context
+            context_parts = [f"Original request: {original_prompt}"]
+            
+            # Add conversation history summary
+            if collected_messages:
+                context_parts.append("\nConversation progress so far:")
+                for i, msg in enumerate(collected_messages[-5:]):  # Last 5 messages for context
+                    if hasattr(msg, 'content'):
+                        context_parts.append(f"- {msg.content[:200]}...")
+            
+            # Add injected messages
+            if injected_messages:
+                context_parts.append("\nAdditional instructions:")
+                for msg in injected_messages:
+                    context_parts.append(f"- {msg}")
+            
+            # Combine into enhanced prompt
+            enhanced_prompt = "\n".join(context_parts)
+            logger.info(f"Built enhanced prompt with {len(injected_messages)} injected messages")
+            
+            return enhanced_prompt
+            
+        except Exception as e:
+            logger.error(f"Error building enhanced prompt: {e}")
+            # Fallback to original prompt
+            return original_prompt
+    
     async def execute_simple(
         self, 
         request: ClaudeCodeRunRequest, 
@@ -151,6 +267,11 @@ class ExecutionStrategies:
         
         logger.info(f"SDK Executor: Starting simple execution for run_id: {run_id}, session: {session_id}")
         
+        # Extract tracing context from agent_context
+        trace_id = agent_context.get("trace_id")
+        parent_span_id = agent_context.get("parent_span_id")
+        tracing = get_tracing_manager() if trace_id else None
+        
         # Initialize metrics handler
         metrics_handler = MetricsHandler()
         
@@ -188,6 +309,22 @@ class ExecutionStrategies:
             # Extract workspace from agent context (normal flow)
             workspace_path = Path(agent_context.get('workspace', '.'))
         
+        # Update database with workspace_path immediately after determining it
+        if hasattr(request, 'run_id') and request.run_id:
+            try:
+                from ...db.models import WorkflowRunUpdate
+                from ...db.repository.workflow_run import update_workflow_run_by_run_id
+                
+                workspace_update = WorkflowRunUpdate(workspace_path=str(workspace_path))
+                update_success = update_workflow_run_by_run_id(request.run_id, workspace_update)
+                if update_success:
+                    logger.info(f"Updated database with workspace_path: {workspace_path}")
+                else:
+                    logger.warning(f"Failed to update workspace_path in database for {request.run_id}")
+                    
+            except Exception as workspace_update_error:
+                logger.error(f"Error updating workspace_path in database: {workspace_update_error}")
+        
         # Build options for SDK
         options = self.build_options(
             workspace_path,
@@ -214,6 +351,13 @@ class ExecutionStrategies:
             
             # Execute SDK query directly (SDK handles max_turns properly)
             try:
+                # Initialize streaming buffer for large responses (especially brain workflow)
+                streaming_buffer = None
+                if hasattr(request, 'workflow_name') and request.workflow_name == 'brain':
+                    from .stream_utils import StreamingBuffer
+                    streaming_buffer = StreamingBuffer(max_chunk_size=16384, max_buffer_size=512000)
+                    logger.info(f"Initialized streaming buffer for brain workflow with 512KB max size")
+                
                 async for message in query(prompt=request.message, options=options):
                     # Check for kill signal before processing each message
                     if hasattr(request, 'run_id') and request.run_id:
@@ -225,8 +369,107 @@ class ExecutionStrategies:
                         except Exception as kill_check_error:
                             logger.error(f"Kill signal check failed: {kill_check_error}")
                     
-                    messages.append(str(message))
-                    collected_messages.append(message)
+                    # Check for pending injected messages
+                    if hasattr(request, 'run_id') and request.run_id and workspace_path:
+                        try:
+                            injected_messages = await self._check_and_process_pending_messages(
+                                workspace_path, request.run_id
+                            )
+                            
+                            if injected_messages:
+                                logger.info(f"📨 Found {len(injected_messages)} injected messages for run {request.run_id}")
+                                
+                                # IMPLEMENT PROPER MESSAGE INJECTION VIA CONVERSATION RESTART
+                                # Instead of just appending to messages, we need to restart the conversation
+                                # with the injected messages included as user input
+                                
+                                # Build enhanced prompt with conversation history + injected messages
+                                enhanced_prompt = await self._build_enhanced_prompt_with_injected_messages(
+                                    request.message, collected_messages, injected_messages
+                                )
+                                
+                                # Log the injection for debugging
+                                if log_writer:
+                                    for injected_msg in injected_messages:
+                                        try:
+                                            await log_writer(
+                                                f"💉 Injecting message: {injected_msg[:100]}...",
+                                                "message_injection"
+                                            )
+                                        except Exception as log_error:
+                                            logger.error(f"Failed to log injected message: {log_error}")
+                                
+                                # Restart conversation with enhanced prompt
+                                logger.info(f"🔄 Restarting conversation with {len(injected_messages)} injected messages")
+                                
+                                # OPTION A: Conversation restart (complex but comprehensive)
+                                # This approach restarts the conversation with injected messages
+                                try:
+                                    # Create new query with enhanced prompt that includes injected messages
+                                    enhanced_request = request
+                                    enhanced_request.message = enhanced_prompt
+                                    
+                                    # Recursively call with enhanced prompt (conversation restart)
+                                    enhanced_result = await self._execute_with_enhanced_context(
+                                        enhanced_request, workspace_path, options, collected_messages, injected_messages
+                                    )
+                                    
+                                    # Return the enhanced result immediately
+                                    if enhanced_result:
+                                        return enhanced_result
+                                        
+                                except Exception as restart_error:
+                                    logger.error(f"Failed to restart conversation with injected messages: {restart_error}")
+                                
+                                # OPTION B: Direct injection (simpler, more reliable)
+                                # Add injected messages as system notices in the conversation flow
+                                logger.info(f"📢 Using direct message injection for {len(injected_messages)} messages")
+                                
+                                # Add injected messages to the conversation as system notices
+                                for i, injected_msg in enumerate(injected_messages):
+                                    injection_notice = f"""
+━━━ NEW USER MESSAGE #{i+1} ━━━
+{injected_msg}
+━━━ END MESSAGE #{i+1} ━━━
+
+Please acknowledge this additional request and incorporate it into your ongoing work.
+"""
+                                    messages.append(injection_notice)
+                                    logger.info(f"💬 Added injected message {i+1} to conversation flow")
+                                    
+                                    # Log the injection
+                                    if log_writer:
+                                        try:
+                                            await log_writer(
+                                                f"💉 DIRECT INJECTION: {injected_msg[:100]}...", 
+                                                "message_injection"
+                                            )
+                                        except Exception as log_error:
+                                            logger.error(f"Failed to log injected message: {log_error}")
+                                
+                        except Exception as message_check_error:
+                            logger.error(f"Failed to check pending messages: {message_check_error}")
+                    
+                    # Process message with streaming buffer for brain workflow
+                    if streaming_buffer:
+                        message_str = str(message)
+                        completed_messages = streaming_buffer.add_chunk(message_str)
+                        
+                        # Add completed messages to our collection
+                        for completed_msg in completed_messages:
+                            messages.append(completed_msg)
+                        
+                        # Always add the raw message to collected_messages for SDK tracking
+                        collected_messages.append(message)
+                        
+                        # Log buffer stats periodically
+                        if len(messages) % 10 == 0:
+                            buffer_stats = streaming_buffer.get_stats()
+                            logger.debug(f"Buffer stats: {buffer_stats}")
+                    else:
+                        # Standard processing for non-brain workflows
+                        messages.append(str(message))
+                        collected_messages.append(message)
                     
                     # Log message to individual workflow log file
                     if log_writer:
@@ -239,6 +482,46 @@ class ExecutionStrategies:
                     if hasattr(message, 'type') and message.type == 'assistant':
                         turn_count += 1
                         
+                        # Log message turn span if tracing is enabled
+                        if tracing and trace_id:
+                            # Get LangWatch provider if available
+                            langwatch_provider = None
+                            if tracing.observability:
+                                for provider in tracing.observability.providers.values():
+                                    if hasattr(provider, 'log_metadata'):
+                                        langwatch_provider = provider
+                                        break
+                            
+                            if langwatch_provider:
+                                turn_span_id = str(uuid4())
+                                langwatch_provider.log_metadata({
+                                    "trace_id": trace_id,
+                                    "span_id": turn_span_id,
+                                    "parent_span_id": parent_span_id,
+                                    "event_type": "span_start",
+                                    "name": f"claude_code.message.turn_{turn_count}",
+                                    "attributes": {
+                                        "turn_number": turn_count,
+                                        "workflow_name": request.workflow_name,
+                                        "message_type": "assistant"
+                                    },
+                                    "timestamp": datetime.utcnow().isoformat()
+                                })
+                                
+                                # Log turn completion with token usage
+                                langwatch_provider.log_metadata({
+                                    "trace_id": trace_id,
+                                    "span_id": turn_span_id,
+                                    "parent_span_id": parent_span_id,
+                                    "event_type": "span_end",
+                                    "name": f"claude_code.message.turn_{turn_count}",
+                                    "attributes": {
+                                        "tokens_used": token_count - last_token_count if 'last_token_count' in locals() else token_count
+                                    },
+                                    "timestamp": datetime.utcnow().isoformat()
+                                })
+                                last_token_count = token_count
+                        
                     if hasattr(message, 'usage') and message.usage:
                         if hasattr(message.usage, 'total_tokens'):
                             token_count = message.usage.total_tokens
@@ -325,6 +608,65 @@ class ExecutionStrategies:
                 # Handle EndOfStream and other streaming errors gracefully
                 if "EndOfStream" in str(stream_error) or "anyio.EndOfStream" in str(type(stream_error)):
                     logger.info("SDK Executor: Stream ended successfully (EndOfStream is normal after completion)")
+                elif "JSONDecodeError" in str(stream_error) or "json.decoder.JSONDecodeError" in str(type(stream_error)):
+                    logger.error(f"SDK Executor: JSON decode error in stream - likely malformed response: {stream_error}")
+                    
+                    # For brain workflow, try to recover using streaming buffer
+                    if streaming_buffer and hasattr(request, 'workflow_name') and request.workflow_name == 'brain':
+                        logger.info("Attempting to recover from JSON decode error using streaming buffer")
+                        try:
+                            # Get any remaining content from buffer
+                            remaining_content = streaming_buffer.get_final_content()
+                            if remaining_content:
+                                logger.info(f"Recovered {len(remaining_content)} chars from streaming buffer")
+                                
+                                # Use brain-specific error handling
+                                from .stream_utils import handle_brain_workflow_json_error
+                                recovery_info = handle_brain_workflow_json_error(
+                                    stream_error, remaining_content, request.workflow_name
+                                )
+                                
+                                # Add recovered content based on what we found
+                                if recovery_info.get("partial_content"):
+                                    partial = recovery_info["partial_content"]
+                                    logger.info(f"Recovered {partial['count']} {partial['type']} from buffer")
+                                    
+                                    # Add partial content as messages
+                                    if partial["type"] == "memory_operations":
+                                        for memory_op in partial["content"]:
+                                            messages.append(f"Recovered memory operation: {memory_op}")
+                                    elif partial["type"] == "yaml_structures":
+                                        for yaml_section in partial["content"]:
+                                            messages.append(f"Recovered YAML section: {yaml_section}")
+                                
+                                elif recovery_info.get("fallback_content"):
+                                    fallback = recovery_info["fallback_content"]
+                                    logger.info(f"Using fallback content: {fallback['type']}")
+                                    messages.append(f"Recovered content: {fallback['content']}")
+                                
+                                else:
+                                    # Even if no structured recovery, add raw content
+                                    messages.append(remaining_content)
+                            
+                            # Get buffer stats for debugging
+                            buffer_stats = streaming_buffer.get_stats()
+                            logger.info(f"Buffer stats: {buffer_stats}")
+                            
+                            # Continue with whatever messages we have
+                            if messages:
+                                logger.info(f"Continuing with {len(messages)} messages after brain workflow recovery")
+                            else:
+                                logger.warning("No messages recovered from brain workflow, raising error")
+                                raise stream_error
+                        except Exception as buffer_error:
+                            logger.error(f"Brain workflow buffer recovery failed: {buffer_error}")
+                            raise stream_error
+                    else:
+                        # Try to continue if we have some messages
+                        if messages:
+                            logger.info(f"Continuing with {len(messages)} messages collected before JSON error")
+                        else:
+                            raise stream_error
                 else:
                     logger.error(f"SDK Executor: Stream error: {stream_error}")
                     raise stream_error
@@ -732,4 +1074,109 @@ class CancellationManager:
             
         except Exception as e:
             logger.error(f"Failed to cancel session {execution_id}: {e}")
-            return False
\ No newline at end of file
+            return False
+
+    async def _build_enhanced_prompt_with_injected_messages(
+        self,
+        original_message: str,
+        conversation_history: List[str],
+        injected_messages: List[str]
+    ) -> str:
+        """
+        Build an enhanced prompt that includes conversation history and injected messages.
+        This creates a new conversation context that incorporates all previous exchanges
+        plus the newly injected user messages.
+        """
+        try:
+            # Build conversation reconstruction
+            enhanced_prompt_parts = []
+            
+            # Add original user message
+            enhanced_prompt_parts.append(f"ORIGINAL REQUEST:\n{original_message}")
+            
+            # Add conversation history if any
+            if conversation_history:
+                enhanced_prompt_parts.append("\nCONVERSATION HISTORY:")
+                for i, msg in enumerate(conversation_history[-5:]):  # Last 5 messages for context
+                    enhanced_prompt_parts.append(f"[{i+1}] {msg}")
+            
+            # Add injected messages as new user input
+            enhanced_prompt_parts.append("\nADDITIONAL USER REQUESTS:")
+            for i, injected_msg in enumerate(injected_messages):
+                enhanced_prompt_parts.append(f"[INJECTED-{i+1}] {injected_msg}")
+            
+            # Add instruction for Claude to handle the enhanced context
+            enhanced_prompt_parts.append("""
+INSTRUCTIONS: 
+Please continue your work taking into account both the original request and the additional user requests above. 
+Integrate the new requirements seamlessly with your current progress. 
+Acknowledge the additional requests and incorporate them into your ongoing work.
+""")
+            
+            enhanced_prompt = "\n".join(enhanced_prompt_parts)
+            logger.debug(f"Built enhanced prompt with {len(injected_messages)} injected messages")
+            
+            return enhanced_prompt
+            
+        except Exception as e:
+            logger.error(f"Failed to build enhanced prompt: {e}")
+            # Fallback to original message with simple injection
+            fallback = f"{original_message}\n\nADDITIONAL REQUESTS: " + " ".join(injected_messages)
+            return fallback
+    
+    async def _execute_with_enhanced_context(
+        self,
+        enhanced_request,
+        workspace_path: Path,
+        options: dict,
+        previous_messages: List[str],
+        injected_messages: List[str]
+    ):
+        """
+        Execute a new Claude conversation with enhanced context that includes injected messages.
+        This effectively restarts the conversation with the injected messages incorporated.
+        """
+        try:
+            logger.info(f"🔄 Starting enhanced conversation with {len(injected_messages)} injected messages")
+            
+            # Create a new SDK execution with the enhanced prompt
+            # This will start a fresh conversation with Claude that includes the injected messages
+            from claude_tools import create_claude_client
+            client = create_claude_client()
+            
+            # Use the enhanced prompt as the new conversation starter
+            enhanced_result = await client.query(
+                enhanced_request.message,
+                **options
+            )
+            
+            # Process the enhanced result
+            enhanced_messages = []
+            enhanced_output = ""
+            
+            async for message in enhanced_result:
+                enhanced_messages.append(str(message))
+                enhanced_output += str(message)
+                
+                # Log the enhanced conversation
+                logger.debug(f"Enhanced conversation message: {str(message)[:100]}...")
+            
+            logger.info(f"✅ Enhanced conversation completed with {len(enhanced_messages)} messages")
+            
+            return {
+                'success': True,
+                'session_id': f"enhanced_{enhanced_request.run_id or 'unknown'}",
+                'result': enhanced_output,
+                'exit_code': 0,
+                'execution_time': time.time() - time.time(),  # Quick execution
+                'logs': f"Enhanced execution with {len(injected_messages)} injected messages",
+                'workspace_path': str(workspace_path),
+                'enhanced_context': True,  # Flag to indicate this was an enhanced execution
+                'injected_message_count': len(injected_messages),
+                'messages': enhanced_messages,
+                'final_output': enhanced_output
+            }
+            
+        except Exception as e:
+            logger.error(f"Enhanced context execution failed: {e}")
+            return None
\ No newline at end of file
