@@ -182,17 +182,30 @@ class PydanticAIFramework(AgentAIFramework):
     def extract_tool_calls(self, result: Any) -> List[Dict[str, Any]]:
         """Extract tool calls from PydanticAI result."""
         tool_calls = []
+        tool_results = {}  # Map tool call IDs to results
         
         try:
             if hasattr(result, 'all_messages'):
+                # First pass: collect tool results from ToolReturnPart
                 for message in result.all_messages():
-                    if hasattr(message, 'tool_calls') and message.tool_calls:
-                        for tool_call in message.tool_calls:
-                            tool_calls.append({
-                                'name': tool_call.function.name,
-                                'arguments': tool_call.function.arguments,
-                                'id': getattr(tool_call, 'id', None)
-                            })
+                    if hasattr(message, 'parts') and message.parts:
+                        for part in message.parts:
+                            # Check if this is a tool return part
+                            if hasattr(part, 'tool_call_id') and hasattr(part, 'content') and 'ToolReturnPart' in str(type(part)):
+                                tool_results[part.tool_call_id] = part.content
+                
+                # Second pass: extract tool calls from ToolCallPart and match with results
+                for message in result.all_messages():
+                    if hasattr(message, 'parts') and message.parts:
+                        for part in message.parts:
+                            # Check if this is a tool call part
+                            if hasattr(part, 'tool_call_id') and hasattr(part, 'tool_name') and hasattr(part, 'args') and 'ToolCallPart' in str(type(part)):
+                                tool_calls.append({
+                                    'name': part.tool_name,
+                                    'args': part.args,
+                                    'id': part.tool_call_id,
+                                    'result': tool_results.get(part.tool_call_id, None)
+                                })
                             
         except Exception as e:
             logger.error(f"Error extracting tool calls: {e}")
