@@ -178,6 +178,16 @@ class WorkflowInfo(BaseModel):
     description: str = Field(..., description="Workflow description")
     path: str = Field(..., description="Path to workflow configuration")
     valid: bool = Field(..., description="Whether the workflow is valid")
+    
+    # New required fields for enhanced workflow metadata
+    icon: str = Field(default="Bot", description="Lucide icon name for the workflow")
+    mainColour: str = Field(default="#3B82F6", description="Primary color for the workflow UI")
+    displayName: str = Field(default="", description="Human-friendly display name")
+    category: str = Field(default="general", description="Workflow category")
+    capabilities: List[str] = Field(default_factory=list, description="List of workflow capabilities")
+    emoji: str = Field(default="🤖", description="Emoji representation of the workflow")
+    maxTurns: Optional[int] = Field(None, description="Maximum conversation turns")
+    suggestedTurns: Optional[int] = Field(None, description="Suggested conversation turns")
 
 
 @claude_code_router.post(
@@ -789,10 +799,10 @@ async def get_claude_code_run_status(run_id: str, debug: bool = False):
 @claude_code_router.get("/manage", response_model=List[WorkflowInfo])
 async def list_claude_code_workflows() -> List[WorkflowInfo]:
     """
-    List all available Claude-Code workflows.
+    List all available Claude-Code workflows with enhanced metadata.
 
     **Returns:**
-    List of available workflows with their descriptions and validation status.
+    List of available workflows with their descriptions, validation status, and UI metadata.
 
     **Example:**
     ```bash
@@ -800,25 +810,35 @@ async def list_claude_code_workflows() -> List[WorkflowInfo]:
     ```
     """
     try:
-        # Get the claude-code agent
-        agent = AgentFactory.get_agent("claude_code")
-        if not agent:
-            raise HTTPException(
-                status_code=404, detail="Claude-Code agent not available"
-            )
-
-        # Get available workflows
-        workflows = await agent.get_available_workflows()
-
-        # Convert to response format
+        # Get workflows from database
+        from automagik.db import list_workflows
+        
+        # Get all workflows (both system and custom)
+        db_workflows = list_workflows(active_only=True)
+        
+        # Convert to response format with enhanced metadata
         workflow_list = []
-        for name, info in workflows.items():
+        
+        for workflow in db_workflows:
+            # Get metadata from config field (stored as JSON in database)
+            config = workflow.config or {}
+            
+            # Build workflow info with all required fields
             workflow_list.append(
                 WorkflowInfo(
-                    name=name,
-                    description=info.get("description", "No description available"),
-                    path=info.get("path", ""),
-                    valid=info.get("valid", False),
+                    name=workflow.name,
+                    description=workflow.description or config.get("description", "No description available"),
+                    path=f"/workflows/{workflow.name}",
+                    valid=True,  # Database workflows are assumed valid
+                    # Enhanced metadata fields from config
+                    icon=config.get("icon", "Bot"),
+                    mainColour=config.get("mainColour", "#3B82F6"),
+                    displayName=config.get("display_name") or workflow.display_name or workflow.name.title(),
+                    category=workflow.category or config.get("category", "general"),
+                    capabilities=config.get("capabilities", []),
+                    emoji=config.get("emoji", "🤖"),
+                    maxTurns=config.get("maxTurns"),
+                    suggestedTurns=config.get("suggestedTurns", 50)
                 )
             )
 
@@ -833,6 +853,209 @@ async def list_claude_code_workflows() -> List[WorkflowInfo]:
         )
 
 
+class InjectMessageRequest(BaseModel):
+    """Request model for injecting a message into a running workflow."""
+    
+    message: str = Field(
+        ...,
+        min_length=1,
+        max_length=10000,
+        description="Message to inject into the running workflow"
+    )
+
+
+@claude_code_router.post("/run/{run_id}/inject-message")
+async def inject_message_to_running_workflow(
+    run_id: str = Path(..., description="Run ID of the active workflow"),
+    request: InjectMessageRequest = Body(...)
+) -> Dict[str, Any]:
+    """
+    Inject a new message into a running Claude Code workflow.
+    
+    This endpoint allows you to send additional messages to an already running
+    workflow without stopping and restarting it. Messages are queued and processed
+    by the workflow in the order they are received.
+    
+    **Parameters:**
+    - `run_id`: The run ID of the active workflow
+    - `message`: The message to inject into the workflow
+    
+    **Returns:**
+    Confirmation that the message was queued for injection.
+    
+    **Examples:**
+    ```bash
+    # Inject a message into a running workflow
+    POST /api/v1/workflows/claude-code/run/abc-123/inject-message
+    {
+        "message": "Please also add error handling to the implementation"
+    }
+    ```
+    
+    **Notes:**
+    - Only works with actively running workflows
+    - Messages are processed in the order they are received
+    - The workflow must be in "running" status
+    - Messages are persisted even if workflow is temporarily paused
+    """
+    try:
+        import json
+        from pathlib import Path as PathLib
+        from datetime import datetime
+        
+        # Verify workflow is running
+        from automagik.db.repository.workflow_run import get_workflow_run_by_run_id
+        
+        workflow_run = get_workflow_run_by_run_id(run_id)
+        if not workflow_run:
+            raise HTTPException(
+                status_code=404, 
+                detail=f"Workflow run {run_id} not found"
+            )
+        
+        if workflow_run.status not in ["running", "pending"]:
+            raise HTTPException(
+                status_code=400,
+                detail=f"Cannot inject message into workflow with status '{workflow_run.status}'. Workflow must be running or pending."
+            )
+        
+        # Handle workspace initialization timing - poll for workspace path
+        workspace_path = workflow_run.workspace_path
+        if not workspace_path:
+            # Workspace not ready yet, implement retry logic for pending workflows
+            if workflow_run.status == "pending":
+                # Wait for workspace initialization with exponential backoff
+                max_retries = 10
+                base_delay = 0.5  # Start with 500ms
+                
+                for attempt in range(max_retries):
+                    await asyncio.sleep(base_delay * (2 ** attempt))  # Exponential backoff
+                    
+                    # Re-fetch workflow run to check for workspace
+                    updated_workflow = get_workflow_run_by_run_id(run_id)
+                    if updated_workflow and updated_workflow.workspace_path:
+                        workspace_path = updated_workflow.workspace_path
+                        break
+                    
+                    # If workflow status changed to running, continue waiting
+                    if updated_workflow and updated_workflow.status == "running":
+                        continue
+                    
+                    # If workflow failed or completed, stop retrying
+                    if updated_workflow and updated_workflow.status in ["failed", "completed", "killed"]:
+                        raise HTTPException(
+                            status_code=400,
+                            detail=f"Workflow {run_id} is no longer active (status: {updated_workflow.status})"
+                        )
+                
+                # If still no workspace after retries
+                if not workspace_path:
+                    raise HTTPException(
+                        status_code=408,
+                        detail=f"Workspace initialization timeout for workflow run {run_id}. Please try again in a few moments."
+                    )
+            else:
+                # For non-pending workflows, workspace should already exist
+                raise HTTPException(
+                    status_code=400,
+                    detail=f"No workspace found for workflow run {run_id}"
+                )
+        
+        workspace_dir = PathLib(workspace_path)
+        if not workspace_dir.exists():
+            raise HTTPException(
+                status_code=400,
+                detail=f"Workspace directory does not exist: {workspace_path}"
+            )
+        
+        # Create message queue file path
+        message_queue_file = workspace_dir / ".pending_messages.json"
+        
+        # Prepare message data
+        message_data = {
+            "id": str(uuid.uuid4()),
+            "message": request.message,
+            "injected_at": datetime.utcnow().isoformat(),
+            "run_id": run_id,
+            "processed": False
+        }
+        
+        # Load existing messages or create new queue
+        existing_messages = []
+        if message_queue_file.exists():
+            try:
+                with open(message_queue_file, 'r') as f:
+                    existing_messages = json.load(f)
+                if not isinstance(existing_messages, list):
+                    existing_messages = []
+            except (json.JSONDecodeError, IOError):
+                existing_messages = []
+        
+        # Add new message to queue (FIFO order)
+        existing_messages.append(message_data)
+        
+        # Write updated message queue
+        try:
+            with open(message_queue_file, 'w') as f:
+                json.dump(existing_messages, f, indent=2)
+            
+            logger.info(f"Injected message into workflow {run_id}: {request.message[:100]}...")
+            
+        except IOError as e:
+            raise HTTPException(
+                status_code=500,
+                detail=f"Failed to write message to workspace: {str(e)}"
+            )
+        
+        # Update workflow run metadata to track message injection
+        try:
+            from automagik.db.models import WorkflowRunUpdate
+            from automagik.db.repository.workflow_run import update_workflow_run_by_run_id
+            
+            # Get existing metadata
+            current_metadata = workflow_run.metadata or {}
+            injected_messages = current_metadata.get("injected_messages", [])
+            
+            # Add message summary to metadata
+            injected_messages.append({
+                "message_id": message_data["id"],
+                "injected_at": message_data["injected_at"],
+                "message_preview": request.message[:100] + ("..." if len(request.message) > 100 else "")
+            })
+            
+            current_metadata["injected_messages"] = injected_messages
+            current_metadata["last_message_injection"] = datetime.utcnow().isoformat()
+            
+            update_data = WorkflowRunUpdate(
+                metadata=current_metadata,
+                updated_at=datetime.utcnow()
+            )
+            update_workflow_run_by_run_id(run_id, update_data)
+            
+        except Exception as metadata_error:
+            logger.warning(f"Failed to update workflow metadata for message injection: {metadata_error}")
+            # Don't fail the injection for metadata issues
+        
+        return {
+            "success": True,
+            "message_id": message_data["id"],
+            "run_id": run_id,
+            "message": "Message queued for injection into running workflow",
+            "queue_position": len(existing_messages),
+            "injected_at": message_data["injected_at"],
+            "workspace_path": str(workspace_path)
+        }
+        
+    except HTTPException:
+        raise
+    except Exception as e:
+        logger.error(f"Error injecting message into workflow {run_id}: {e}")
+        raise HTTPException(
+            status_code=500,
+            detail=f"Failed to inject message: {str(e)}"
+        )
+
+
 @claude_code_router.post("/run/{run_id}/kill")
 async def kill_claude_code_run(
     run_id: str = Path(..., description="Run ID to terminate"),
@@ -1055,6 +1278,14 @@ class WorkflowManageRequest(BaseModel):
     allowed_tools: List[str] = Field(default_factory=list, description="List of allowed tool names")
     mcp_config: Dict[str, Any] = Field(default_factory=dict, description="MCP server configuration")
     active: bool = Field(default=True, description="Whether workflow is active")
+    
+    # Enhanced metadata fields
+    icon: Optional[str] = Field(default="Bot", description="Lucide icon name")
+    mainColour: Optional[str] = Field(default="#3B82F6", description="Primary color hex code")
+    emoji: Optional[str] = Field(default="🤖", description="Emoji representation")
+    capabilities: Optional[List[str]] = Field(default_factory=list, description="List of capabilities")
+    maxTurns: Optional[int] = Field(None, description="Maximum conversation turns")
+    suggestedTurns: Optional[int] = Field(default=50, description="Suggested conversation turns")
     config: Dict[str, Any] = Field(default_factory=dict, description="Additional configuration")
 
 
@@ -1090,6 +1321,17 @@ async def create_workflow(request: WorkflowManageRequest) -> WorkflowManageRespo
         
         # No category validation - allow any category
         
+        # Prepare enhanced config with metadata
+        enhanced_config = request.config.copy()
+        enhanced_config.update({
+            "icon": request.icon,
+            "mainColour": request.mainColour,
+            "emoji": request.emoji,
+            "capabilities": request.capabilities,
+            "maxTurns": request.maxTurns,
+            "suggestedTurns": request.suggestedTurns
+        })
+        
         # Create workflow
         workflow_create = WorkflowCreate(
             name=request.name,
@@ -1101,7 +1343,7 @@ async def create_workflow(request: WorkflowManageRequest) -> WorkflowManageRespo
             mcp_config=request.mcp_config,
             active=request.active,
             is_system_workflow=False,  # Custom workflows are never system workflows
-            config=request.config
+            config=enhanced_config
         )
         
         workflow_id = create_workflow(workflow_create)
@@ -1111,6 +1353,8 @@ async def create_workflow(request: WorkflowManageRequest) -> WorkflowManageRespo
             from automagik.db import get_workflow
             workflow = get_workflow(workflow_id)
             
+            # Include enhanced metadata in response
+            config = workflow.config or {}
             return WorkflowManageResponse(
                 success=True,
                 message=f"Workflow '{request.name}' created successfully",
@@ -1122,7 +1366,14 @@ async def create_workflow(request: WorkflowManageRequest) -> WorkflowManageRespo
                     "category": workflow.category,
                     "active": workflow.active,
                     "is_system_workflow": workflow.is_system_workflow,
-                    "created_at": workflow.created_at.isoformat() if hasattr(workflow.created_at, 'isoformat') else str(workflow.created_at)
+                    "created_at": workflow.created_at.isoformat() if hasattr(workflow.created_at, 'isoformat') else str(workflow.created_at),
+                    # Enhanced metadata
+                    "icon": config.get("icon", "Bot"),
+                    "mainColour": config.get("mainColour", "#3B82F6"),
+                    "emoji": config.get("emoji", "🤖"),
+                    "capabilities": config.get("capabilities", []),
+                    "maxTurns": config.get("maxTurns"),
+                    "suggestedTurns": config.get("suggestedTurns", 50)
                 }
             )
         else:
@@ -1166,6 +1417,17 @@ async def update_workflow(request: WorkflowManageRequest) -> WorkflowManageRespo
         
         # No category validation - allow any category
         
+        # Prepare enhanced config with metadata
+        enhanced_config = request.config.copy()
+        enhanced_config.update({
+            "icon": request.icon,
+            "mainColour": request.mainColour,
+            "emoji": request.emoji,
+            "capabilities": request.capabilities,
+            "maxTurns": request.maxTurns,
+            "suggestedTurns": request.suggestedTurns
+        })
+        
         # Update workflow
         workflow_update = WorkflowUpdate(
             display_name=request.display_name,
@@ -1175,7 +1437,7 @@ async def update_workflow(request: WorkflowManageRequest) -> WorkflowManageRespo
             allowed_tools=request.allowed_tools,
             mcp_config=request.mcp_config,
             active=request.active,
-            config=request.config
+            config=enhanced_config
         )
         
         success = update_workflow(existing_workflow.id, workflow_update)
@@ -1185,6 +1447,8 @@ async def update_workflow(request: WorkflowManageRequest) -> WorkflowManageRespo
             from automagik.db import get_workflow
             workflow = get_workflow(existing_workflow.id)
             
+            # Include enhanced metadata in response
+            config = workflow.config or {}
             return WorkflowManageResponse(
                 success=True,
                 message=f"Workflow '{request.name}' updated successfully",
@@ -1196,7 +1460,14 @@ async def update_workflow(request: WorkflowManageRequest) -> WorkflowManageRespo
                     "category": workflow.category,
                     "active": workflow.active,
                     "is_system_workflow": workflow.is_system_workflow,
-                    "updated_at": workflow.updated_at.isoformat() if hasattr(workflow.updated_at, 'isoformat') else str(workflow.updated_at)
+                    "updated_at": workflow.updated_at.isoformat() if hasattr(workflow.updated_at, 'isoformat') else str(workflow.updated_at),
+                    # Enhanced metadata
+                    "icon": config.get("icon", "Bot"),
+                    "mainColour": config.get("mainColour", "#3B82F6"),
+                    "emoji": config.get("emoji", "🤖"),
+                    "capabilities": config.get("capabilities", []),
+                    "maxTurns": config.get("maxTurns"),
+                    "suggestedTurns": config.get("suggestedTurns", 50)
                 }
             )
         else:
