@@ -62,19 +62,35 @@ async def initialize_all_agents():
         for agent_name in available_agents:
             existing_agent = get_agent_by_name(agent_name)
             if not existing_agent:
+                # Get agent class to read DEFAULT_MODEL and DEFAULT_CONFIG
+                try:
+                    agent_class = AgentFactory.get_agent_class(agent_name)
+                    if agent_class and hasattr(agent_class, 'DEFAULT_MODEL'):
+                        model = agent_class.DEFAULT_MODEL
+                        default_config = getattr(agent_class, 'DEFAULT_CONFIG', {})
+                    else:
+                        # Fall back to creating temporary instance to get model
+                        temp_agent = AgentFactory.create_agent(agent_name, {})
+                        model = getattr(temp_agent.config, 'model', 'openai:gpt-4o-mini')
+                        default_config = {}
+                except Exception as e:
+                    logger.warning(f"Could not get model for {agent_name}: {e}")
+                    model = "openai:gpt-4o-mini"  # Sensible default
+                    default_config = {}
+                
                 # Create new agent in database
                 from automagik.agents.models.framework_types import FrameworkType
                 new_agent = Agent(
                     name=agent_name,
                     type=FrameworkType.default().value,  # Use enum for consistency
-                    model="openai:gpt-4.1",  # Default model
-                    config={"created_by": "auto_discovery"},
+                    model=model,  # Use agent's declared model
+                    config={**default_config, "created_by": "auto_discovery"},
                     description=f"Auto-discovered {agent_name} agent",
                     active=True  # Default to active
                 )
                 create_agent(new_agent)
                 registered_count += 1
-                logger.info(f"📝 Registered new agent in database: {agent_name}")
+                logger.info(f"📝 Registered new agent in database: {agent_name} with model {model}")
         
         if registered_count > 0:
             logger.info(f"✅ Registered {registered_count} new agents in database")
@@ -252,6 +268,18 @@ def create_app() -> FastAPI:
             raise e
         
         
+        # Initialize tracing system
+        try:
+            logger.info("📊 Initializing tracing system...")
+            from automagik.tracing import get_tracing_manager
+            tracing = get_tracing_manager()
+            if tracing:
+                logger.info(f"✅ Tracing system initialized - Telemetry: {tracing.telemetry is not None}, Observability: {tracing.observability is not None}")
+            else:
+                logger.info("📊 Tracing system disabled or not configured")
+        except Exception as e:
+            logger.warning(f"⚠️ Could not initialize tracing system: {e}")
+        
         # Initialize agents after core services are ready
         await initialize_all_agents()
         
@@ -349,6 +377,14 @@ def create_app() -> FastAPI:
         allow_headers=["*"],  # Allows all headers
     )
 
+    # Add request logging middleware
+    try:
+        from automagik.api.middleware import RequestLoggingMiddleware
+        app.add_middleware(RequestLoggingMiddleware)
+        logger.info("✅ Added request logging middleware")
+    except Exception as e:
+        logger.warning(f"⚠️ Failed to add request logging middleware: {str(e)}")
+
     # Add JSON parsing middleware to fix malformed JSON
     try:
         from automagik.api.middleware import JSONParsingMiddleware
@@ -360,6 +396,14 @@ def create_app() -> FastAPI:
     # Add authentication middleware
     app.add_middleware(APIKeyMiddleware)
     
+    # Add tracing middleware if available
+    try:
+        from automagik.tracing.middleware import TracingMiddleware
+        app.add_middleware(TracingMiddleware)
+        logger.info("✅ Added tracing middleware")
+    except ImportError:
+        logger.debug("Tracing middleware not available, skipping")
+    
     # Set up database message store regardless of environment
     try:
         logger.info("🔧 Initializing database connection for message storage")
@@ -449,6 +493,13 @@ def create_app() -> FastAPI:
             response = await call_next(request)
             return response
         except Exception as exc:
+            # Handle client disconnects gracefully without logging as errors
+            from starlette.requests import ClientDisconnect
+            if isinstance(exc, ClientDisconnect):
+                logger.debug(f"Client disconnected from {request.url}")
+                # Return a basic response that won't be sent anyway since client disconnected
+                return JSONResponse(status_code=499, content={"detail": "Client disconnected"})
+            
             # Log the error with traceback so we can diagnose pre-router failures
             logger.error(f"❌ Unhandled exception in request {request.url}: {exc}")
             logger.error(f"❌ Traceback: {traceback.format_exc()}")
